@@ -18,22 +18,22 @@
 
 #include <sstream>
 
-#include <ignition/common/Console.hh>
-#include <ignition/common/Material.hh>
-#include <ignition/common/MeshManager.hh>
-#include <ignition/common/Skeleton.hh>
-#include <ignition/common/SkeletonAnimation.hh>
-#include <ignition/common/SubMesh.hh>
+#include <gz/common/Console.hh>
+#include <gz/common/Material.hh>
+#include <gz/common/MeshManager.hh>
+#include <gz/common/Skeleton.hh>
+#include <gz/common/SkeletonAnimation.hh>
+#include <gz/common/SubMesh.hh>
 
-#include <ignition/math/Matrix4.hh>
+#include <gz/math/Matrix4.hh>
 
-#include "ignition/rendering/ogre2/Ogre2Conversions.hh"
-#include "ignition/rendering/ogre2/Ogre2Mesh.hh"
-#include "ignition/rendering/ogre2/Ogre2MeshFactory.hh"
-#include "ignition/rendering/ogre2/Ogre2RenderEngine.hh"
-#include "ignition/rendering/ogre2/Ogre2RenderTypes.hh"
-#include "ignition/rendering/ogre2/Ogre2Scene.hh"
-#include "ignition/rendering/ogre2/Ogre2Storage.hh"
+#include "gz/rendering/ogre2/Ogre2Conversions.hh"
+#include "gz/rendering/ogre2/Ogre2Mesh.hh"
+#include "gz/rendering/ogre2/Ogre2MeshFactory.hh"
+#include "gz/rendering/ogre2/Ogre2RenderEngine.hh"
+#include "gz/rendering/ogre2/Ogre2RenderTypes.hh"
+#include "gz/rendering/ogre2/Ogre2Scene.hh"
+#include "gz/rendering/ogre2/Ogre2Storage.hh"
 
 #ifdef _MSC_VER
   #pragma warning(push, 0)
@@ -56,12 +56,15 @@
 #endif
 
 /// \brief Private data for the Ogre2MeshFactory class
-class ignition::rendering::Ogre2MeshFactoryPrivate
+class gz::rendering::Ogre2MeshFactoryPrivate
 {
+  /// \brief Vector with the template materials, we keep the pointer to be
+  /// able to remove it when nobody is using it.
+  public: std::vector<MaterialPtr> materialCache;
 };
 
 /// \brief Private data for the Ogre2SubMeshStoreFactory class
-class ignition::rendering::Ogre2SubMeshStoreFactoryPrivate
+class gz::rendering::Ogre2SubMeshStoreFactoryPrivate
 {
 };
 
@@ -88,6 +91,25 @@ void Ogre2MeshFactory::Clear()
   this->ogreMeshes.clear();
 }
 
+//////////////////////////////////////////////////
+void Ogre2MeshFactory::ClearMaterialsCache(const std::string &_name)
+{
+  auto it = this->dataPtr->materialCache.begin();
+  for (auto &mat : this->dataPtr->materialCache)
+  {
+    std::string matName = mat->Name();
+    std::string textureName = mat->Texture();
+    if (textureName == _name)
+    {
+      this->scene->UnregisterMaterial(matName);
+      break;
+    }
+    ++it;
+  }
+  if (it != this->dataPtr->materialCache.end())
+    this->dataPtr->materialCache.erase(it);
+}
+
 //////////////////////////////////////////////////
 Ogre2MeshPtr Ogre2MeshFactory::Create(const MeshDescriptor &_desc)
 {
@@ -108,6 +130,12 @@ Ogre2MeshPtr Ogre2MeshFactory::Create(const MeshDescriptor &_desc)
   // create sub-mesh store
   Ogre2SubMeshStoreFactory subMeshFactory(this->scene, mesh->ogreItem);
   mesh->subMeshes = subMeshFactory.Create();
+  for (unsigned int i = 0; i < mesh->subMeshes->Size(); i++)
+  {
+    Ogre2SubMeshPtr submesh =
+        std::dynamic_pointer_cast<Ogre2SubMesh>(mesh->subMeshes->GetById(i));
+    submesh->SetMeshName(this->MeshName(_desc));
+  }
   return mesh;
 }
 
@@ -447,6 +475,7 @@ bool Ogre2MeshFactory::LoadImpl(const MeshDescriptor &_desc)
       if (material)
       {
         mat->CopyFrom(*material);
+        this->dataPtr->materialCache.push_back(mat);
       }
       else
       {
