@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2020 Open Source Robotics Foundation
+ * Copyright (C) 2022 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -22,11 +22,11 @@
 
 #include "test_config.h"  // NOLINT(build/include)
 
-#include "ignition/rendering/ArrowVisual.hh"
-#include "ignition/rendering/RenderEngine.hh"
-#include "ignition/rendering/RenderingIface.hh"
-#include "ignition/rendering/Scene.hh"
-#include "ignition/rendering/Visual.hh"
+#include "gz/rendering/ArrowVisual.hh"
+#include "gz/rendering/RenderEngine.hh"
+#include "gz/rendering/RenderingIface.hh"
+#include "gz/rendering/Scene.hh"
+#include "gz/rendering/Visual.hh"
 
 using namespace ignition;
 using namespace rendering;
@@ -34,19 +34,26 @@ using namespace rendering;
 class ArrowVisualTest : public testing::Test,
                         public testing::WithParamInterface<const char *>
 {
-  /// \brief Test adding removing children
+  /// \brief Test basic API
   public: void ArrowVisual(const std::string &_renderEngine);
+
+  /// \brief Test gizmo material
+  public: void Material(const std::string &_renderEngine);
 };
 
 /////////////////////////////////////////////////
 void ArrowVisualTest::ArrowVisual(const std::string &_renderEngine)
 {
+#ifdef __APPLE__                                                                                                                                                                              
+  std::cerr << "Skipping test for apple, see issue #847." << std::endl;
+  return;
+#endif
+
   RenderEngine *engine = rendering::engine(_renderEngine);
   if (!engine)
   {
-    igndbg << "Engine '" << _renderEngine
-              << "' is not supported" << std::endl;
-    return;
+    FAIL() << "Engine '" << _renderEngine
+      << "' is not supported" << std::endl;
   }
 
   ScenePtr scene = engine->CreateScene("scene");
@@ -64,20 +71,32 @@ void ArrowVisualTest::ArrowVisual(const std::string &_renderEngine)
   EXPECT_EQ(math::Vector3d(0.2, 0.3, 0.4), visual->LocalScale());
 
   // check children and geometry
-  EXPECT_EQ(2u, visual->ChildCount());
+  EXPECT_EQ(3u, visual->ChildCount());
 
   NodePtr node = visual->ChildByIndex(0u);
   VisualPtr child = std::dynamic_pointer_cast<Visual>(node);
   ASSERT_NE(nullptr, child);
   EXPECT_EQ(1u, child->GeometryCount());
-  EXPECT_EQ(node, visual->Shaft());
+  EXPECT_EQ(node, visual->Rotation());
 
   node = visual->ChildByIndex(1u);
   child = std::dynamic_pointer_cast<Visual>(node);
   ASSERT_NE(nullptr, child);
   EXPECT_EQ(1u, child->GeometryCount());
+  EXPECT_EQ(node, visual->Shaft());
+
+  node = visual->ChildByIndex(2u);
+  child = std::dynamic_pointer_cast<Visual>(node);
+  ASSERT_NE(nullptr, child);
+  EXPECT_EQ(1u, child->GeometryCount());
   EXPECT_EQ(node, visual->Head());
 
+  // test destroy
+  ArrowVisualPtr visual2 = scene->CreateArrowVisual();
+  ASSERT_NE(nullptr, visual2);
+  visual2->Destroy();
+  EXPECT_EQ(0u, visual2->ChildCount());
+
   // Clean up
   engine->DestroyScene(scene);
   rendering::unloadEngine(engine->Name());
