@@ -35,18 +35,25 @@ using namespace rendering;
 class AxisVisualTest : public testing::Test,
                        public testing::WithParamInterface<const char *>
 {
-  /// \brief Test adding removing children
+  /// \brief Test basic API
   public: void AxisVisual(const std::string &_renderEngine);
+
+  /// \brief Test gizmo material
+  public: void Material(const std::string &_renderEngine);
 };
 
 /////////////////////////////////////////////////
 void AxisVisualTest::AxisVisual(const std::string &_renderEngine)
 {
+#ifdef __APPLE__                                                                              
+  std::cerr << "Skipping test for apple, see issue #847." << std::endl;                       
+  return;
+#endif
   RenderEngine *engine = rendering::engine(_renderEngine);
   if (!engine)
   {
-    igndbg << "Engine '" << _renderEngine
-              << "' is not supported" << std::endl;
+    FAIL() << "Engine '" << _renderEngine
+           << "' is not supported" << std::endl;
     return;
   }
 
@@ -73,7 +80,7 @@ void AxisVisualTest::AxisVisual(const std::string &_renderEngine)
     ArrowVisualPtr arrow = std::dynamic_pointer_cast<ArrowVisual>(node);
     ASSERT_NE(nullptr, arrow);
 
-    EXPECT_EQ(2u, arrow->ChildCount());
+    EXPECT_EQ(3u, arrow->ChildCount());
     NodePtr childNode = arrow->ChildByIndex(0u);
     VisualPtr child = std::dynamic_pointer_cast<Visual>(childNode);
     EXPECT_EQ(1u, child->GeometryCount());
@@ -81,8 +88,19 @@ void AxisVisualTest::AxisVisual(const std::string &_renderEngine)
     childNode = arrow->ChildByIndex(1u);
     child = std::dynamic_pointer_cast<Visual>(childNode);
     EXPECT_EQ(1u, child->GeometryCount());
+
+    childNode = arrow->ChildByIndex(2u);
+    child = std::dynamic_pointer_cast<Visual>(childNode);
+    EXPECT_EQ(1u, child->GeometryCount());
   }
 
+  // create visual
+  AxisVisualPtr axis_name = scene->CreateAxisVisual("axis_name");
+  EXPECT_NE(nullptr, axis_name);
+
+  AxisVisualPtr axis_id = scene->CreateAxisVisual(101);
+  EXPECT_NE(nullptr, axis_id);
+
   // Clean up
   engine->DestroyScene(scene);
   rendering::unloadEngine(engine->Name());
