@@ -17,17 +17,18 @@
 
 #include <gtest/gtest.h>
 
-#include <ignition/common/Console.hh>
+#include <gz/common/Console.hh>
 
 #include "test_config.h"  // NOLINT(build/include)
-#include "ignition/rendering/Camera.hh"
-#include "ignition/rendering/GaussianNoisePass.hh"
-#include "ignition/rendering/RenderEngine.hh"
-#include "ignition/rendering/RenderingIface.hh"
-#include "ignition/rendering/RenderPassSystem.hh"
-#include "ignition/rendering/Scene.hh"
+#include "gz/rendering/Camera.hh"
+#include "gz/rendering/GaussianNoisePass.hh"
+#include "gz/rendering/RenderEngine.hh"
+#include "gz/rendering/RenderingIface.hh"
+#include "gz/rendering/RenderPassSystem.hh"
+#include "gz/rendering/Scene.hh"
+#include "gz/rendering/Utils.hh"
 
-using namespace ignition;
+using namespace gz;
 using namespace rendering;
 
 class CameraTest : public testing::Test,
@@ -47,16 +48,23 @@ class CameraTest : public testing::Test,
 
   /// \brief Test setting visibility mask
   public: void VisibilityMask(const std::string &_renderEngine);
+
+  /// \brief Test setting intrinsic matrix
+  public: void IntrinsicMatrix(const std::string &_renderEngine);
 };
 
 /////////////////////////////////////////////////
 void CameraTest::ViewProjectionMatrix(const std::string &_renderEngine)
 {
+#ifdef __APPLE__                                                                              
+  std::cerr << "Skipping test for apple, see issue #847." << std::endl;                       
+  return;
+#endif
   // create and populate scene
   RenderEngine *engine = rendering::engine(_renderEngine);
   if (!engine)
   {
-    igndbg << "Engine '" << _renderEngine
+    FAIL() << "Engine '" << _renderEngine
               << "' is not supported" << std::endl;
     return;
   }
@@ -72,7 +80,7 @@ void CameraTest::ViewProjectionMatrix(const std::string &_renderEngine)
   EXPECT_GT(camera->HFOV(), 0);
   math::Angle hfov(1.57);
   camera->SetHFOV(hfov);
-  EXPECT_DOUBLE_EQ(hfov.Radian(), camera->HFOV().Radian());
+  EXPECT_NEAR(hfov.Radian(), camera->HFOV().Radian(), 1e-6);
 
   EXPECT_GT(camera->AspectRatio(), 0);
   camera->SetAspectRatio(1.7777);
@@ -105,19 +113,65 @@ void CameraTest::ViewProjectionMatrix(const std::string &_renderEngine)
 
   EXPECT_NE(viewMatrix, camera->ViewMatrix());
 
+  // projection type
+  math::Matrix4d initialProjectionMatrix = camera->ProjectionMatrix();
+  math::Matrix4d initialViewMatrix = camera->ViewMatrix();
+  EXPECT_EQ(CameraProjectionType::CPT_PERSPECTIVE, camera->ProjectionType());
+  camera->SetProjectionType(CameraProjectionType::CPT_ORTHOGRAPHIC);
+  EXPECT_EQ(CameraProjectionType::CPT_ORTHOGRAPHIC, camera->ProjectionType());
+  EXPECT_NE(initialProjectionMatrix, camera->ProjectionMatrix());
+  EXPECT_EQ(initialViewMatrix, camera->ViewMatrix());
+
+  camera->SetProjectionType(CameraProjectionType::CPT_PERSPECTIVE);
+  EXPECT_EQ(CameraProjectionType::CPT_PERSPECTIVE, camera->ProjectionType());
+  EXPECT_EQ(initialProjectionMatrix, camera->ProjectionMatrix());
+  EXPECT_EQ(initialViewMatrix, camera->ViewMatrix());
+
+  // project 3d to 2d
+  unsigned int width = 320u;
+  unsigned int height = 240u;
+  camera->SetImageWidth(width);
+  camera->SetImageHeight(height);
+  camera->SetLocalPosition(math::Vector3d::Zero);
+  camera->SetLocalRotation(math::Quaterniond::Identity);
+
+  math::Vector2i pos2d = camera->Project(math::Vector3d(2.0, 0, 0));
+  EXPECT_EQ(static_cast<int>(width * 0.5), pos2d.X());
+  EXPECT_EQ(static_cast<int>(height * 0.5), pos2d.Y());
+
+  pos2d = camera->Project(math::Vector3d(3.0, 0, 0));
+  EXPECT_EQ(static_cast<int>(width * 0.5), pos2d.X());
+  EXPECT_EQ(static_cast<int>(height * 0.5), pos2d.Y());
+
+  pos2d = camera->Project(math::Vector3d(2.0, 1, 0));
+  EXPECT_GT(static_cast<int>(width * 0.5), pos2d.X());
+  EXPECT_EQ(static_cast<int>(height * 0.5), pos2d.Y());
+
+  pos2d = camera->Project(math::Vector3d(2.0, 1, 1));
+  EXPECT_GT(static_cast<int>(width * 0.5), pos2d.X());
+  EXPECT_GT(static_cast<int>(height * 0.5), pos2d.Y());
+
+  pos2d = camera->Project(math::Vector3d(2.0, -1, -1));
+  EXPECT_LT(static_cast<int>(width * 0.5), pos2d.X());
+  EXPECT_LT(static_cast<int>(height * 0.5), pos2d.Y());
+
   // Clean up
   engine->DestroyScene(scene);
-  rendering::unloadEngine(engine->Name());
+  unloadEngine(engine->Name());
 }
 
 /////////////////////////////////////////////////
 void CameraTest::RenderTexture(const std::string &_renderEngine)
 {
+#ifdef __APPLE__                                                                              
+  std::cerr << "Skipping test for apple, see issue #847." << std::endl;                       
+  return;
+#endif
   // create and populate scene
   RenderEngine *engine = rendering::engine(_renderEngine);
   if (!engine)
   {
-    igndbg << "Engine '" << _renderEngine
+    FAIL() << "Engine '" << _renderEngine
               << "' is not supported" << std::endl;
     return;
   }
@@ -129,17 +183,26 @@ void CameraTest::RenderTexture(const std::string &_renderEngine)
 
   // render texture parameters
   EXPECT_GT(camera->ImageWidth(), 0u);
-  camera->SetImageWidth(100u);
-  EXPECT_EQ(100u, camera->ImageWidth());
-
   EXPECT_GT(camera->ImageHeight(), 0u);
-  camera->SetImageHeight(80u);
-  EXPECT_EQ(80u, camera->ImageHeight());
+
+  unsigned int height = 80;
+  camera->SetImageHeight(height);
+  EXPECT_EQ(height, camera->ImageHeight());
+  double aspectRatio =
+    static_cast<double>(camera->ImageWidth()) / static_cast<double>(height);
+  EXPECT_NEAR(aspectRatio, camera->AspectRatio(), 1e-6);
+
+  unsigned int width = 100;
+  camera->SetImageWidth(width);
+  EXPECT_EQ(width, camera->ImageWidth());
+  aspectRatio =
+    static_cast<double>(width) / static_cast<double>(camera->ImageHeight());
+  EXPECT_NEAR(aspectRatio, camera->AspectRatio(), 1e-6);
 
   EXPECT_NE(PixelFormat::PF_UNKNOWN, camera->ImageFormat());
   camera->SetImageFormat(PixelFormat::PF_B8G8R8);
   EXPECT_EQ(PixelFormat::PF_B8G8R8, camera->ImageFormat());
-  EXPECT_EQ(100u*80u*3u, camera->ImageMemorySize());
+  EXPECT_EQ(width*height*3u, camera->ImageMemorySize());
 
 
   // verify render texture GL Id
@@ -152,17 +215,21 @@ void CameraTest::RenderTexture(const std::string &_renderEngine)
 
   // Clean up
   engine->DestroyScene(scene);
-  rendering::unloadEngine(engine->Name());
+  unloadEngine(engine->Name());
 }
 
 /////////////////////////////////////////////////
 void CameraTest::TrackFollow(const std::string &_renderEngine)
 {
+#ifdef __APPLE__                                                                              
+  std::cerr << "Skipping test for apple, see issue #847." << std::endl;                       
+  return;
+#endif
   // create and populate scene
   RenderEngine *engine = rendering::engine(_renderEngine);
   if (!engine)
   {
-    igndbg << "Engine '" << _renderEngine
+    FAIL() << "Engine '" << _renderEngine
               << "' is not supported" << std::endl;
     return;
   }
@@ -224,17 +291,21 @@ void CameraTest::TrackFollow(const std::string &_renderEngine)
 
   // Clean up
   engine->DestroyScene(scene);
-  rendering::unloadEngine(engine->Name());
+  unloadEngine(engine->Name());
 }
 
 /////////////////////////////////////////////////
 void CameraTest::AddRemoveRenderPass(const std::string &_renderEngine)
 {
+#ifdef __APPLE__                                                                              
+  std::cerr << "Skipping test for apple, see issue #847." << std::endl;                       
+  return;
+#endif
   // create and populate scene
   RenderEngine *engine = rendering::engine(_renderEngine);
   if (!engine)
   {
-    igndbg << "Engine '" << _renderEngine
+    FAIL() << "Engine '" << _renderEngine
               << "' is not supported" << std::endl;
     return;
   }
@@ -278,17 +349,21 @@ void CameraTest::AddRemoveRenderPass(const std::string &_renderEngine)
 
   // Clean up
   engine->DestroyScene(scene);
-  rendering::unloadEngine(engine->Name());
+  unloadEngine(engine->Name());
 }
 
 /////////////////////////////////////////////////
 void CameraTest::VisibilityMask(const std::string &_renderEngine)
 {
+#ifdef __APPLE__                                                                              
+  std::cerr << "Skipping test for apple, see issue #847." << std::endl;                       
+  return;
+#endif
   // create and populate scene
   RenderEngine *engine = rendering::engine(_renderEngine);
   if (!engine)
   {
-    igndbg << "Engine '" << _renderEngine
+    FAIL() << "Engine '" << _renderEngine
               << "' is not supported" << std::endl;
     return;
   }
@@ -314,6 +389,86 @@ void CameraTest::VisibilityMask(const std::string &_renderEngine)
   rendering::unloadEngine(engine->Name());
 }
 
+/////////////////////////////////////////////////
+void CameraTest::IntrinsicMatrix(const std::string &_renderEngine)
+{
+#ifdef __APPLE__                                                                              
+  std::cerr << "Skipping test for apple, see issue #847." << std::endl;                       
+  return;
+#endif
+  // create and populate scene
+  RenderEngine *engine = rendering::engine(_renderEngine);
+  if (!engine)
+  {
+    FAIL() << "Engine '" << _renderEngine
+              << "' is not supported" << std::endl;
+    return;
+  }
+
+  ScenePtr scene = engine->CreateScene("scene");
+  ASSERT_NE(nullptr, scene);
+
+  CameraPtr camera = scene->CreateCamera();
+  EXPECT_TRUE(camera != nullptr);
+
+  unsigned int width = 320;
+  unsigned int height = 240;
+  double hfov = 1.047;
+
+  camera->SetImageHeight(height);
+  camera->SetImageWidth(width);
+  camera->SetHFOV(hfov);
+
+  double error = 1e-1;
+  EXPECT_EQ(camera->ImageHeight(), height);
+  EXPECT_EQ(camera->ImageWidth(), width);
+  EXPECT_NEAR(camera->HFOV().Radian(), hfov, error);
+
+  // Verify focal length and optical center from intrinsics
+  auto cameraIntrinsics = projectionToCameraIntrinsic(
+      camera->ProjectionMatrix(),
+      camera->ImageWidth(),
+      camera->ImageHeight()
+    );
+  EXPECT_NEAR(cameraIntrinsics(0, 0), 277.1913, error);
+  EXPECT_NEAR(cameraIntrinsics(1, 1), 277.1913, error);
+  EXPECT_DOUBLE_EQ(cameraIntrinsics(0, 2), 160);
+  EXPECT_DOUBLE_EQ(cameraIntrinsics(1, 2), 120);
+
+  // Verify rest of the intrinsics
+  EXPECT_DOUBLE_EQ(cameraIntrinsics(0, 1), 0);
+  EXPECT_DOUBLE_EQ(cameraIntrinsics(1, 0), 0);
+  EXPECT_DOUBLE_EQ(cameraIntrinsics(0, 1), 0);
+  EXPECT_DOUBLE_EQ(cameraIntrinsics(2, 0), 0);
+  EXPECT_DOUBLE_EQ(cameraIntrinsics(2, 1), 0);
+  EXPECT_DOUBLE_EQ(cameraIntrinsics(2, 2), 1);
+
+  // Verify that changing camera size changes intrinsics
+  height = 1000;
+  width = 1000;
+  camera->SetImageHeight(height);
+  camera->SetImageWidth(width);
+  camera->SetHFOV(hfov);
+
+  EXPECT_EQ(camera->ImageHeight(), height);
+  EXPECT_EQ(camera->ImageWidth(), width);
+  EXPECT_NEAR(camera->HFOV().Radian(), hfov, error);
+
+  // Verify if intrinsics have changed
+  cameraIntrinsics = projectionToCameraIntrinsic(
+      camera->ProjectionMatrix(),
+      camera->ImageWidth(),
+      camera->ImageHeight()
+    );
+  EXPECT_NEAR(cameraIntrinsics(0, 0), 866.223, error);
+  EXPECT_NEAR(cameraIntrinsics(1, 1), 866.223, error);
+  EXPECT_DOUBLE_EQ(cameraIntrinsics(0, 2), 500);
+  EXPECT_DOUBLE_EQ(cameraIntrinsics(1, 2), 500);
+
+  // Clean up
+  engine->DestroyScene(scene);
+}
+
 /////////////////////////////////////////////////
 TEST_P(CameraTest, ViewProjectionMatrix)
 {
@@ -344,9 +499,15 @@ TEST_P(CameraTest, VisibilityMask)
   VisibilityMask(GetParam());
 }
 
+/////////////////////////////////////////////////
+TEST_P(CameraTest, IntrinsicMatrix)
+{
+  IntrinsicMatrix(GetParam());
+}
+
 INSTANTIATE_TEST_CASE_P(Camera, CameraTest,
     RENDER_ENGINE_VALUES,
-    ignition::rendering::PrintToStringParam());
+    PrintToStringParam());
 
 int main(int argc, char **argv)
 {
