@@ -37,6 +37,10 @@ class LidarVisualTest : public testing::Test,
 /////////////////////////////////////////////////
 void LidarVisualTest::LidarVisual(const std::string &_renderEngine)
 {
+#ifdef __APPLE__                                                                                                                                                                              
+  std::cerr << "Skipping test for apple, see issue #847." << std::endl;
+  return;
+#endif
   if (_renderEngine == "optix")
   {
     igndbg << "LidarVisual not supported yet in rendering engine: "
@@ -48,7 +52,7 @@ void LidarVisualTest::LidarVisual(const std::string &_renderEngine)
   RenderEngine *engine = rendering::engine(_renderEngine);
   if (!engine)
   {
-    igndbg << "Engine '" << _renderEngine
+    FAIL() << "Engine '" << _renderEngine
            << "' is not supported" << std::endl;
     return;
   }
@@ -120,6 +124,9 @@ void LidarVisualTest::LidarVisual(const std::string &_renderEngine)
                           10, 3.5};
   lidar->SetPoints(pts);
   EXPECT_EQ(pts.size(), lidar->PointCount());
+
+  lidar->PreRender();
+
   lidar->ClearPoints();
   EXPECT_EQ(lidar->PointCount(), 0u);
 
