@@ -36,6 +36,12 @@ using namespace rendering;
 class MaterialTest : public testing::Test,
                      public testing::WithParamInterface<const char *>
 {
+  // Documentation inherited
+  public: void SetUp() override
+  {
+    ignition::common::Console::SetVerbosity(4);
+  }
+
   /// \brief Test material basic API
   public: void MaterialProperties(const std::string &_renderEngine);
 
@@ -50,10 +56,14 @@ class MaterialTest : public testing::Test,
 /////////////////////////////////////////////////
 void MaterialTest::MaterialProperties(const std::string &_renderEngine)
 {
+#ifdef __APPLE__                                                                              
+  std::cerr << "Skipping test for apple, see issue #847." << std::endl;                       
+  return;
+#endif
   RenderEngine *engine = rendering::engine(_renderEngine);
   if (!engine)
   {
-    igndbg << "Engine '" << _renderEngine
+    FAIL() << "Engine '" << _renderEngine
               << "' is not supported" << std::endl;
     return;
   }
@@ -113,6 +123,16 @@ void MaterialTest::MaterialProperties(const std::string &_renderEngine)
   material->SetTransparency(transparency);
   EXPECT_DOUBLE_EQ(transparency, material->Transparency());
 
+  // alpha from texture
+  bool alphaFromTexture = true;
+  double alphaThreshold = 0.9;
+  bool twoSidedEnabled = false;
+  material->SetAlphaFromTexture(alphaFromTexture, alphaThreshold,
+      twoSidedEnabled);
+  EXPECT_EQ(material->TextureAlphaEnabled(), alphaFromTexture);
+  EXPECT_DOUBLE_EQ(material->AlphaThreshold(), alphaThreshold);
+  EXPECT_EQ(material->TwoSidedEnabled(), twoSidedEnabled);
+
   // reflectivity
   double reflectivity = 0.5;
   material->SetReflectivity(reflectivity);
@@ -235,6 +255,21 @@ void MaterialTest::MaterialProperties(const std::string &_renderEngine)
     EXPECT_EQ(noSuchEmissiveMapName, material->EmissiveMap());
     EXPECT_TRUE(material->HasEmissiveMap());
 
+    // light map
+    std::string lightMapName = textureName;
+    material->SetLightMap(lightMapName, 1u);
+    EXPECT_EQ(lightMapName, material->LightMap());
+    EXPECT_EQ(1u, material->LightMapTexCoordSet());
+    EXPECT_TRUE(material->HasLightMap());
+
+    material->ClearLightMap();
+    EXPECT_FALSE(material->HasLightMap());
+
+    std::string noSuchLightMapName = "no_such_light.png";
+    material->SetLightMap(noSuchLightMapName);
+    EXPECT_EQ(noSuchLightMapName, material->LightMap());
+    EXPECT_TRUE(material->HasLightMap());
+
     // roughness
     float roughness = 0.3f;
     material->SetRoughness(roughness);
@@ -259,10 +294,14 @@ void MaterialTest::MaterialProperties(const std::string &_renderEngine)
 /////////////////////////////////////////////////
 void MaterialTest::Copy(const std::string &_renderEngine)
 {
+#ifdef __APPLE__                                                                              
+  std::cerr << "Skipping test for apple, see issue #847." << std::endl;                       
+  return;
+#endif
   RenderEngine *engine = rendering::engine(_renderEngine);
   if (!engine)
   {
-    igndbg << "Engine '" << _renderEngine
+    FAIL() << "Engine '" << _renderEngine
               << "' is not supported" << std::endl;
     return;
   }
@@ -278,6 +317,9 @@ void MaterialTest::Copy(const std::string &_renderEngine)
   math::Color emissive(0.6f, 0.4f, 0.2f, 1.0f);
   double shininess = 0.8;
   double transparency = 0.3;
+  bool alphaFromTexture = true;
+  double alphaThreshold = 0.9;
+  bool twoSidedEnabled = false;
   double reflectivity = 0.5;
   bool castShadows = false;
   bool receiveShadows = false;
@@ -295,6 +337,7 @@ void MaterialTest::Copy(const std::string &_renderEngine)
   std::string metalnessMapName = "metalness_" + textureName;
   std::string envMapName = "env_" + textureName;
   std::string emissiveMapName = "emissive_" + textureName;
+  std::string lightMapName = "light_" + textureName;
   enum ShaderType shaderType = ShaderType::ST_PIXEL;
 
   material->SetAmbient(ambient);
@@ -303,6 +346,8 @@ void MaterialTest::Copy(const std::string &_renderEngine)
   material->SetEmissive(emissive);
   material->SetShininess(shininess);
   material->SetTransparency(transparency);
+  material->SetAlphaFromTexture(alphaFromTexture, alphaThreshold,
+      twoSidedEnabled);
   material->SetReflectivity(reflectivity);
   material->SetCastShadows(castShadows);
   material->SetReceiveShadows(receiveShadows);
@@ -317,6 +362,7 @@ void MaterialTest::Copy(const std::string &_renderEngine)
   material->SetMetalnessMap(metalnessMapName);
   material->SetEnvironmentMap(envMapName);
   material->SetEmissiveMap(emissiveMapName);
+  material->SetLightMap(lightMapName, 1u);
   material->SetRoughness(roughness);
   material->SetMetalness(metalness);
 
@@ -329,6 +375,9 @@ void MaterialTest::Copy(const std::string &_renderEngine)
   EXPECT_EQ(emissive, clone->Emissive());
   EXPECT_DOUBLE_EQ(shininess, clone->Shininess());
   EXPECT_DOUBLE_EQ(transparency, clone->Transparency());
+  EXPECT_EQ(alphaFromTexture, clone->TextureAlphaEnabled());
+  EXPECT_DOUBLE_EQ(alphaThreshold, clone->AlphaThreshold());
+  EXPECT_EQ(twoSidedEnabled, clone->TwoSidedEnabled());
   EXPECT_DOUBLE_EQ(reflectivity, clone->Reflectivity());
   EXPECT_EQ(castShadows, clone->CastShadows());
   EXPECT_EQ(receiveShadows, clone->ReceiveShadows());
@@ -349,6 +398,8 @@ void MaterialTest::Copy(const std::string &_renderEngine)
     EXPECT_EQ(metalnessMapName, clone->MetalnessMap());
     EXPECT_EQ(envMapName, clone->EnvironmentMap());
     EXPECT_EQ(emissiveMapName, clone->EmissiveMap());
+    EXPECT_EQ(lightMapName, clone->LightMap());
+    EXPECT_EQ(1u, clone->LightMapTexCoordSet());
   }
 
   // test copying a material
@@ -361,6 +412,9 @@ void MaterialTest::Copy(const std::string &_renderEngine)
   EXPECT_EQ(emissive, copy->Emissive());
   EXPECT_DOUBLE_EQ(shininess, copy->Shininess());
   EXPECT_DOUBLE_EQ(transparency, copy->Transparency());
+  EXPECT_EQ(alphaFromTexture, copy->TextureAlphaEnabled());
+  EXPECT_DOUBLE_EQ(alphaThreshold, copy->AlphaThreshold());
+  EXPECT_EQ(twoSidedEnabled, copy->TwoSidedEnabled());
   EXPECT_DOUBLE_EQ(reflectivity, copy->Reflectivity());
   EXPECT_EQ(castShadows, copy->CastShadows());
   EXPECT_EQ(receiveShadows, copy->ReceiveShadows());
@@ -379,17 +433,21 @@ void MaterialTest::Copy(const std::string &_renderEngine)
     EXPECT_EQ(metalnessMapName, copy->MetalnessMap());
     EXPECT_EQ(envMapName, copy->EnvironmentMap());
     EXPECT_EQ(emissiveMapName, copy->EmissiveMap());
+    EXPECT_EQ(lightMapName, copy->LightMap());
+    EXPECT_EQ(1u, copy->LightMapTexCoordSet());
   }
 
   // test copying from a common material
   // common::Material currently only has a subset of material properties
   common::Material comMat;
   comMat.SetAmbient(ambient);
-  comMat.SetDiffuse(ambient);
-  comMat.SetSpecular(ambient);
-  comMat.SetEmissive(ambient);
+  comMat.SetDiffuse(diffuse);
+  comMat.SetSpecular(specular);
+  comMat.SetEmissive(emissive);
   comMat.SetShininess(shininess);
   comMat.SetTransparency(transparency);
+  comMat.SetAlphaFromTexture(alphaFromTexture, alphaThreshold,
+      twoSidedEnabled);
   comMat.SetLighting(lightingEnabled);
   comMat.SetTextureImage(textureName);
   common::Pbr pbr;
@@ -402,18 +460,23 @@ void MaterialTest::Copy(const std::string &_renderEngine)
   pbr.SetMetalnessMap(metalnessMapName);
   pbr.SetEmissiveMap(emissiveMapName);
   pbr.SetEnvironmentMap(envMapName);
+  pbr.SetLightMap(lightMapName, 1u);
   comMat.SetPbrMaterial(pbr);
 
   MaterialPtr comCopy = scene->CreateMaterial("comCopy");
   EXPECT_TRUE(scene->MaterialRegistered("comCopy"));
-  comCopy->CopyFrom(material);
+  comCopy->CopyFrom(comMat);
   EXPECT_EQ(ambient, comCopy->Ambient());
   EXPECT_EQ(diffuse, comCopy->Diffuse());
   EXPECT_EQ(specular, comCopy->Specular());
   EXPECT_EQ(emissive, comCopy->Emissive());
   EXPECT_DOUBLE_EQ(shininess, comCopy->Shininess());
   EXPECT_DOUBLE_EQ(transparency, comCopy->Transparency());
-  EXPECT_DOUBLE_EQ(reflectivity, comCopy->Reflectivity());
+  EXPECT_EQ(alphaFromTexture, comCopy->TextureAlphaEnabled());
+  EXPECT_DOUBLE_EQ(alphaThreshold, comCopy->AlphaThreshold());
+  EXPECT_EQ(twoSidedEnabled, comCopy->TwoSidedEnabled());
+  // \todo(anyon) add reflectivity to common::Material?
+  // EXPECT_DOUBLE_EQ(reflectivity, comCopy->Reflectivity());
   EXPECT_EQ(lightingEnabled, comCopy->LightingEnabled());
   EXPECT_EQ(textureName, comCopy->Texture());
   EXPECT_TRUE(comCopy->HasTexture());
@@ -429,6 +492,9 @@ void MaterialTest::Copy(const std::string &_renderEngine)
     EXPECT_EQ(metalnessMapName, comCopy->MetalnessMap());
     EXPECT_TRUE(comCopy->HasEmissiveMap());
     EXPECT_EQ(emissiveMapName, comCopy->EmissiveMap());
+    EXPECT_TRUE(comCopy->HasLightMap());
+    EXPECT_EQ(lightMapName, comCopy->LightMap());
+    EXPECT_EQ(1u, comCopy->LightMapTexCoordSet());
     EXPECT_TRUE(comCopy->HasEnvironmentMap());
     EXPECT_EQ(envMapName, comCopy->EnvironmentMap());
   }
