@@ -17,17 +17,17 @@
 
 #include <gtest/gtest.h>
 
-#include <ignition/common/Console.hh>
+#include <gz/common/Console.hh>
 
 #include "test_config.h"  // NOLINT(build/include)
 
-#include "ignition/rendering/Camera.hh"
-#include "ignition/rendering/RayQuery.hh"
-#include "ignition/rendering/RenderEngine.hh"
-#include "ignition/rendering/RenderingIface.hh"
-#include "ignition/rendering/Scene.hh"
+#include "gz/rendering/Camera.hh"
+#include "gz/rendering/RayQuery.hh"
+#include "gz/rendering/RenderEngine.hh"
+#include "gz/rendering/RenderingIface.hh"
+#include "gz/rendering/Scene.hh"
 
-using namespace ignition;
+using namespace gz;
 using namespace rendering;
 
 class RayQueryTest : public testing::Test,
@@ -40,6 +40,10 @@ class RayQueryTest : public testing::Test,
 /////////////////////////////////////////////////
 void RayQueryTest::RayQuery(const std::string &_renderEngine)
 {
+#ifdef __APPLE__                                                                              
+  std::cerr << "Skipping test for apple, see issue #847." << std::endl;                       
+  return;
+#endif
   if (_renderEngine == "optix")
   {
     igndbg << "RayQuery not supported yet in rendering engine: "
@@ -51,7 +55,7 @@ void RayQueryTest::RayQuery(const std::string &_renderEngine)
   RenderEngine *engine = rendering::engine(_renderEngine);
   if (!engine)
   {
-    igndbg << "Engine '" << _renderEngine
+    FAIL() << "Engine '" << _renderEngine
               << "' is not supported" << std::endl;
     return;
   }
@@ -88,21 +92,49 @@ void RayQueryTest::RayQuery(const std::string &_renderEngine)
 
   // set from camera
   CameraPtr camera =  scene->CreateCamera("camera");
+  camera->SetLocalPosition(math::Vector3d::Zero);
+  unsigned int width = 320u;
+  unsigned int height = 240u;
+  camera->SetImageWidth(width);
+  camera->SetImageHeight(height);
+
+  VisualPtr root = scene->RootVisual();
+  root->AddChild(camera);
   math::Vector2d pos(0.0, 0.0);
   rayQuery->SetFromCamera(camera, pos);
 
-  EXPECT_GT(rayQuery->Origin().X(), 0.0);
+  EXPECT_LT(0.0, rayQuery->Origin().X());
   EXPECT_EQ(math::Vector3d::UnitX, rayQuery->Direction().Normalize());
 
   RayQueryResult result = rayQuery->ClosestPoint();
   EXPECT_EQ(math::Vector3d::Zero, result.point);
-  EXPECT_LT(result.distance, 0.0);
+  EXPECT_GT(0.0, result.distance);
   EXPECT_EQ(0u, result.objectId);
-  EXPECT_FALSE((result));
+  EXPECT_FALSE(result);
+
+  // testing multiple ray queries that are set from camera
+  CameraPtr camera2 =  scene->CreateCamera("camera2");
+  camera2->SetLocalPosition(math::Vector3d::Zero);
+  camera2->SetImageWidth(width);
+  camera2->SetImageHeight(height);
+
+  root->AddChild(camera2);
+  math::Vector2d pos2(0.0, 0.0);
+  RayQueryPtr rayQuery2 = scene->CreateRayQuery();
+  rayQuery2->SetFromCamera(camera2, pos2);
+
+  EXPECT_LT(0.0, rayQuery2->Origin().X());
+  EXPECT_EQ(math::Vector3d::UnitX, rayQuery2->Direction().Normalize());
+
+  RayQueryResult result2 = rayQuery2->ClosestPoint();
+  EXPECT_EQ(math::Vector3d::Zero, result2.point);
+  EXPECT_GT(0.0, result2.distance);
+  EXPECT_EQ(0u, result2.objectId);
+  EXPECT_FALSE(result2);
 
   // Clean up
   engine->DestroyScene(scene);
-  rendering::unloadEngine(engine->Name());
+  unloadEngine(engine->Name());
 }
 
 /////////////////////////////////////////////////
@@ -113,7 +145,7 @@ TEST_P(RayQueryTest, RayQuery)
 
 INSTANTIATE_TEST_CASE_P(RayQuery, RayQueryTest,
     RENDER_ENGINE_VALUES,
-    ignition::rendering::PrintToStringParam());
+    PrintToStringParam());
 
 int main(int argc, char **argv)
 {
