@@ -48,11 +48,15 @@ class RenderTargetTest : public testing::Test,
 /////////////////////////////////////////////////
 void RenderTargetTest::RenderTexture(const std::string &_renderEngine)
 {
+#ifdef __APPLE__                                                                              
+  std::cerr << "Skipping test for apple, see issue #847." << std::endl;                       
+  return;
+#endif
   // create and populate scene
   RenderEngine *engine = rendering::engine(_renderEngine);
   if (!engine)
   {
-    igndbg << "Engine '" << _renderEngine
+    FAIL() << "Engine '" << _renderEngine
               << "' is not supported" << std::endl;
     return;
   }
@@ -81,6 +85,10 @@ void RenderTargetTest::RenderTexture(const std::string &_renderEngine)
 /////////////////////////////////////////////////
 void RenderTargetTest::RenderWindow(const std::string &_renderEngine)
 {
+#ifdef __APPLE__                                                                                                                                                                              
+  std::cerr << "Skipping test for apple, see issue #847." << std::endl;
+  return;
+#endif
   if (_renderEngine != "ogre")
   {
     igndbg << "RenderWindow not supported yet in rendering engine: "
@@ -92,7 +100,7 @@ void RenderTargetTest::RenderWindow(const std::string &_renderEngine)
   RenderEngine *engine = rendering::engine(_renderEngine);
   if (!engine)
   {
-    igndbg << "Engine '" << _renderEngine
+    FAIL() << "Engine '" << _renderEngine
               << "' is not supported" << std::endl;
     return;
   }
@@ -130,6 +138,10 @@ void RenderTargetTest::RenderWindow(const std::string &_renderEngine)
 /////////////////////////////////////////////////
 void RenderTargetTest::AddRemoveRenderPass(const std::string &_renderEngine)
 {
+#ifdef __APPLE__                                                                                                                                                                              
+  std::cerr << "Skipping test for apple, see issue #847." << std::endl;
+  return;
+#endif
   if (_renderEngine != "ogre")
   {
     igndbg << "RenderWindow not supported yet in rendering engine: "
@@ -141,7 +153,7 @@ void RenderTargetTest::AddRemoveRenderPass(const std::string &_renderEngine)
   RenderEngine *engine = rendering::engine(_renderEngine);
   if (!engine)
   {
-    igndbg << "Engine '" << _renderEngine
+    FAIL() << "Engine '" << _renderEngine
               << "' is not supported" << std::endl;
     return;
   }
@@ -179,6 +191,10 @@ void RenderTargetTest::AddRemoveRenderPass(const std::string &_renderEngine)
   renderTexture->RemoveRenderPass(pass1);
   EXPECT_EQ(1u, renderTexture->RenderPassCount());
   EXPECT_EQ(pass2, renderTexture->RenderPassByIndex(0u));
+
+  // Clean up
+  engine->DestroyScene(scene);
+  rendering::unloadEngine(engine->Name());
 }
 
 /////////////////////////////////////////////////
@@ -199,7 +215,6 @@ TEST_P(RenderTargetTest, AddRemoveRenderPass)
   AddRemoveRenderPass(GetParam());
 }
 
-
 INSTANTIATE_TEST_CASE_P(RenderTarget, RenderTargetTest,
     RENDER_ENGINE_VALUES,
     PrintToStringParam());
