@@ -17,34 +17,39 @@
 
 #include <sstream>
 
-#include <ignition/math/Helpers.hh>
+#include <gz/math/Helpers.hh>
 
-#include <ignition/common/Console.hh>
-#include <ignition/common/Mesh.hh>
-#include <ignition/common/MeshManager.hh>
+#include <gz/common/Console.hh>
+#include <gz/common/Mesh.hh>
+#include <gz/common/MeshManager.hh>
 
-#include "ignition/common/Time.hh"
+#include "gz/common/Time.hh"
 
-#include "ignition/rendering/ArrowVisual.hh"
-#include "ignition/rendering/AxisVisual.hh"
-#include "ignition/rendering/LidarVisual.hh"
-#include "ignition/rendering/LightVisual.hh"
-#include "ignition/rendering/Camera.hh"
-#include "ignition/rendering/Capsule.hh"
-#include "ignition/rendering/DepthCamera.hh"
-#include "ignition/rendering/GizmoVisual.hh"
-#include "ignition/rendering/GpuRays.hh"
-#include "ignition/rendering/Grid.hh"
-#include "ignition/rendering/ParticleEmitter.hh"
-#include "ignition/rendering/RayQuery.hh"
-#include "ignition/rendering/RenderTarget.hh"
-#include "ignition/rendering/Text.hh"
-#include "ignition/rendering/ThermalCamera.hh"
-#include "ignition/rendering/Visual.hh"
-#include "ignition/rendering/base/BaseStorage.hh"
-#include "ignition/rendering/base/BaseScene.hh"
+#include "gz/rendering/ArrowVisual.hh"
+#include "gz/rendering/AxisVisual.hh"
+#include "gz/rendering/BoundingBoxCamera.hh"
+#include "gz/rendering/COMVisual.hh"
+#include "gz/rendering/InertiaVisual.hh"
+#include "gz/rendering/JointVisual.hh"
+#include "gz/rendering/LidarVisual.hh"
+#include "gz/rendering/LightVisual.hh"
+#include "gz/rendering/Camera.hh"
+#include "gz/rendering/Capsule.hh"
+#include "gz/rendering/DepthCamera.hh"
+#include "gz/rendering/GizmoVisual.hh"
+#include "gz/rendering/GpuRays.hh"
+#include "gz/rendering/Grid.hh"
+#include "gz/rendering/ParticleEmitter.hh"
+#include "gz/rendering/RayQuery.hh"
+#include "gz/rendering/RenderTarget.hh"
+#include "gz/rendering/Text.hh"
+#include "gz/rendering/ThermalCamera.hh"
+#include "gz/rendering/SegmentationCamera.hh"
+#include "gz/rendering/Visual.hh"
+#include "gz/rendering/base/BaseStorage.hh"
+#include "gz/rendering/base/BaseScene.hh"
 
-using namespace ignition;
+using namespace gz;
 using namespace rendering;
 
 // Prevent deprecation warnings for simTime
@@ -58,7 +63,7 @@ BaseScene::BaseScene(unsigned int _id, const std::string &_name) :
   name(_name),
   loaded(false),
   initialized(false),
-  nextObjectId(ignition::math::MAX_UI16),
+  nextObjectId(math::MAX_UI16),
   nodes(nullptr)
 {
 }
@@ -136,7 +141,13 @@ std::chrono::steady_clock::duration BaseScene::Time() const
 //////////////////////////////////////////////////
 void BaseScene::SetTime(const std::chrono::steady_clock::duration &_time)
 {
-  this->time = _time;
+#if IGNITION_RENDERING_MAJOR_VERSION <= 6
+  // TODO(anyone): Remove me in ign-rendering7
+  if (std::chrono::duration_cast<std::chrono::nanoseconds>(_time).count() != -1)
+#endif
+  {
+    this->time = _time;
+  }
 }
 
 //////////////////////////////////////////////////
@@ -160,7 +171,7 @@ void BaseScene::SetSimTime(const common::Time &_time)
 
 //////////////////////////////////////////////////
 VisualPtr BaseScene::VisualAt(const CameraPtr &_camera,
-                              const ignition::math::Vector2i &_mousePos)
+                              const math::Vector2i &_mousePos)
 {
   VisualPtr visual;
   RayQueryPtr rayQuery = this->CreateRayQuery();
@@ -811,6 +822,66 @@ ThermalCameraPtr BaseScene::CreateThermalCamera(const unsigned int _id,
   return (result) ? camera : nullptr;
 }
 
+//////////////////////////////////////////////////
+BoundingBoxCameraPtr BaseScene::CreateBoundingBoxCamera()
+{
+  unsigned int objId = this->CreateObjectId();
+  return this->CreateBoundingBoxCamera(objId);
+}
+//////////////////////////////////////////////////
+BoundingBoxCameraPtr BaseScene::CreateBoundingBoxCamera(
+  const unsigned int _id)
+{
+  std::string objName = this->CreateObjectName(_id, "BoundingBoxCamera");
+  return this->CreateBoundingBoxCamera(_id, objName);
+}
+//////////////////////////////////////////////////
+BoundingBoxCameraPtr BaseScene::CreateBoundingBoxCamera(
+  const std::string &_name)
+{
+  unsigned int objId = this->CreateObjectId();
+  return this->CreateBoundingBoxCamera(objId, _name);
+}
+//////////////////////////////////////////////////
+BoundingBoxCameraPtr BaseScene::CreateBoundingBoxCamera(
+  const unsigned int _id,
+    const std::string &_name)
+{
+  BoundingBoxCameraPtr camera = this->CreateBoundingBoxCameraImpl(_id, _name);
+  bool result = this->RegisterSensor(camera);
+  return (result) ? camera : nullptr;
+}
+
+//////////////////////////////////////////////////
+SegmentationCameraPtr BaseScene::CreateSegmentationCamera()
+{
+  unsigned int objId = this->CreateObjectId();
+  return this->CreateSegmentationCamera(objId);
+}
+//////////////////////////////////////////////////
+SegmentationCameraPtr BaseScene::CreateSegmentationCamera(
+  const unsigned int _id)
+{
+  std::string objName = this->CreateObjectName(_id, "SegmentationCamera");
+  return this->CreateSegmentationCamera(_id, objName);
+}
+//////////////////////////////////////////////////
+SegmentationCameraPtr BaseScene::CreateSegmentationCamera(
+  const std::string &_name)
+{
+  unsigned int objId = this->CreateObjectId();
+  return this->CreateSegmentationCamera(objId, _name);
+}
+//////////////////////////////////////////////////
+SegmentationCameraPtr BaseScene::CreateSegmentationCamera(
+  const unsigned int _id,
+    const std::string &_name)
+{
+  SegmentationCameraPtr camera = this->CreateSegmentationCameraImpl(_id, _name);
+  bool result = this->RegisterSensor(camera);
+  return (result) ? camera : nullptr;
+}
+
 //////////////////////////////////////////////////
 GpuRaysPtr BaseScene::CreateGpuRays()
 {
@@ -918,6 +989,95 @@ AxisVisualPtr BaseScene::CreateAxisVisual(unsigned int _id,
   return (result) ? visual : nullptr;
 }
 
+//////////////////////////////////////////////////
+COMVisualPtr BaseScene::CreateCOMVisual()
+{
+  unsigned int objId = this->CreateObjectId();
+  return this->CreateCOMVisual(objId);
+}
+
+//////////////////////////////////////////////////
+COMVisualPtr BaseScene::CreateCOMVisual(unsigned int _id)
+{
+  std::string objName = this->CreateObjectName(_id, "COMVisual");
+  return this->CreateCOMVisual(_id, objName);
+}
+
+//////////////////////////////////////////////////
+COMVisualPtr BaseScene::CreateCOMVisual(const std::string &_name)
+{
+  unsigned int objId = this->CreateObjectId();
+  return this->CreateCOMVisual(objId, _name);
+}
+
+//////////////////////////////////////////////////
+COMVisualPtr BaseScene::CreateCOMVisual(unsigned int _id,
+    const std::string &_name)
+{
+  COMVisualPtr visual = this->CreateCOMVisualImpl(_id, _name);
+  bool result = this->RegisterVisual(visual);
+  return (result) ? visual : nullptr;
+}
+
+InertiaVisualPtr BaseScene::CreateInertiaVisual()
+{
+  unsigned int objId = this->CreateObjectId();
+  return this->CreateInertiaVisual(objId);
+}
+
+//////////////////////////////////////////////////
+InertiaVisualPtr BaseScene::CreateInertiaVisual(unsigned int _id)
+{
+  std::string objName = this->CreateObjectName(_id, "InertiaVisual");
+  return this->CreateInertiaVisual(_id, objName);
+}
+
+//////////////////////////////////////////////////
+InertiaVisualPtr BaseScene::CreateInertiaVisual(const std::string &_name)
+{
+  unsigned int objId = this->CreateObjectId();
+  return this->CreateInertiaVisual(objId, _name);
+}
+
+//////////////////////////////////////////////////
+InertiaVisualPtr BaseScene::CreateInertiaVisual(unsigned int _id,
+    const std::string &_name)
+{
+  InertiaVisualPtr visual = this->CreateInertiaVisualImpl(_id, _name);
+  bool result = this->RegisterVisual(visual);
+  return (result) ? visual : nullptr;
+}
+
+//////////////////////////////////////////////////
+JointVisualPtr BaseScene::CreateJointVisual()
+{
+  unsigned int objId = this->CreateObjectId();
+  return this->CreateJointVisual(objId);
+}
+
+//////////////////////////////////////////////////
+JointVisualPtr BaseScene::CreateJointVisual(unsigned int _id)
+{
+  std::string objName = this->CreateObjectName(_id, "JointVisual");
+  return this->CreateJointVisual(_id, objName);
+}
+
+//////////////////////////////////////////////////
+JointVisualPtr BaseScene::CreateJointVisual(const std::string &_name)
+{
+  unsigned int objId = this->CreateObjectId();
+  return this->CreateJointVisual(objId, _name);
+}
+
+//////////////////////////////////////////////////
+JointVisualPtr BaseScene::CreateJointVisual(unsigned int _id,
+    const std::string &_name)
+{
+  JointVisualPtr visual = this->CreateJointVisualImpl(_id, _name);
+  bool result = this->RegisterVisual(visual);
+  return (result) ? visual : nullptr;
+}
+
 //////////////////////////////////////////////////
 LightVisualPtr BaseScene::CreateLightVisual()
 {
@@ -1235,12 +1395,40 @@ void BaseScene::PreRender()
   this->RootVisual()->PreRender();
 }
 
+//////////////////////////////////////////////////
+void BaseScene::PostRender()
+{
+}
+
+//////////////////////////////////////////////////
+void BaseScene::SetCameraPassCountPerGpuFlush(uint8_t /*_numPass*/)
+{
+}
+
+//////////////////////////////////////////////////
+uint8_t BaseScene::CameraPassCountPerGpuFlush() const
+{
+  return 0u;
+}
+
+//////////////////////////////////////////////////
+bool BaseScene::LegacyAutoGpuFlush() const
+{
+  return true;
+}
+
 //////////////////////////////////////////////////
 void BaseScene::Clear()
 {
-  this->nodes->DestroyAll();
+  this->DestroyNodes();
+  auto root = this->RootVisual();
+  if (root)
+  {
+    root->RemoveChildren();
+    this->DestroyNode(root);
+  }
   this->DestroyMaterials();
-  this->nextObjectId = ignition::math::MAX_UI16;
+  this->nextObjectId = math::MAX_UI16;
 }
 
 //////////////////////////////////////////////////
@@ -1350,6 +1538,16 @@ void BaseScene::CreateMaterials()
   material->SetReceiveShadows(false);
   material->SetLightingEnabled(false);
 
+  material = this->CreateMaterial("Default/TransPurple");
+  material->SetAmbient(1.0, 0.0, 1.0);
+  material->SetDiffuse(1.0, 0.0, 1.0);
+  material->SetEmissive(1.0, 0.0, 1.0);
+  material->SetTransparency(0.5);
+  material->SetCastShadows(false);
+  material->SetReceiveShadows(false);
+  material->SetLightingEnabled(false);
+  material->SetDepthWriteEnabled(false);
+
   material = this->CreateMaterial("Default/White");
   material->SetAmbient(1.0, 1.0, 1.0);
   material->SetDiffuse(1.0, 1.0, 1.0);
@@ -1358,4 +1556,18 @@ void BaseScene::CreateMaterials()
   material->SetCastShadows(true);
   material->SetReceiveShadows(true);
   material->SetLightingEnabled(true);
+
+  const char *env = std::getenv("IGN_RENDERING_RESOURCE_PATH");
+  std::string resourcePath = (env) ? std::string(env) :
+      IGN_RENDERING_RESOURCE_PATH;
+
+  // path to look for CoM material texture
+  std::string com_material_texture_path = common::joinPaths(
+      resourcePath, "media", "materials", "textures", "com.png");
+  material = this->CreateMaterial("Default/CoM");
+  material->SetTexture(com_material_texture_path);
+  material->SetTransparency(0);
+  material->SetCastShadows(false);
+  material->SetReceiveShadows(true);
+  material->SetLightingEnabled(true);
 }
